Imports Syncfusion.Windows.Forms.Grid
Imports Syncfusion.Windows.Forms.Chart
Imports Syncfusion.Windows.Forms.Tools
Imports Syncfusion.Drawing


Imports System.Drawing
Imports System.Drawing.Color


Public Class Form

    Private _dataCollection As DataCollection
    Private _listOfColumnLables As List(Of String)

    Private _longLabelText As Boolean = False


    Private Sub Form_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

        _dataCollection = New DataCollection

        _updateButton.Enabled = False

        InitializeChart()

    End Sub


    ''' <summary>
    ''' Initializes the column chart
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub InitializeChart()

        _barChart.BeginUpdate()

        'Name of chart
        _barChart.Text = "Column labels"


        'Color palette
        _barChart.Palette = ChartColorPalette.Custom
        _barChart.Palette = ChartColorPalette.Custom

        'Hide the gridlines
        _barChart.PrimaryXAxis.DrawGrid = False

        'Set user mode for the tick labels to be able to have both integers and strings of the tick labels
        _barChart.PrimaryXAxis.TickLabelsDrawingMode = ChartAxisTickLabelDrawingMode.UserMode

        'Background color
        _barChart.ChartInterior = New Syncfusion.Drawing.BrushInfo(185, New Syncfusion.Drawing.BrushInfo(Syncfusion.Drawing.GradientStyle.None, Color.White, Color.White))

        'Hide labels for all columns (only show the labels for a whole group)
        _barChart.ChartArea.HidePartialLabels = True

        'Hide legend (there are no correspondence between series and groups anyway)
        _barChart.Legend.Hide()

        'Hide gridlines
        _barChart.PrimaryXAxis.DrawGrid = False

        'Zooming
        _barChart.EnableXZooming = True
        _barChart.EnableYZooming = True

        'Column widths
        _barChart.ColumnWidthMode = ChartColumnWidthMode.RelativeWidthMode
        _barChart.ColumnDrawMode = ChartColumnDrawMode.InDepthMode
        _barChart.Spacing = 15

        'Show Tooltips
        _barChart.ShowToolTips = True

        _barChart.EndUpdate()

    End Sub



    ''' <summary>
    ''' Updates the chart.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub UpdateChart()
        Dim series As ChartSeries
        Dim dataItem As DataItem
        Dim dataCollection As List(Of DataItem)
        Dim columnIndex As Integer
        Dim maxXValue, minXValue As Integer
        Dim maxYValue, minYValue As Double
        Dim firstTime As Boolean = True
        Dim labelsHasBeenShown As Boolean = False
        Dim pointIndex As Integer
        Dim displayText As String

        _barChart.BeginUpdate()


        'Clear chart and labels
        _barChart.Series.Clear()
        _barChart.PrimaryYAxis.Labels.Clear()
        _barChart.PrimaryXAxis.Labels.Clear()

        'Set name and title of x-axis
        _barChart.PrimaryYAxis.Title = "y-axis"
        _barChart.PrimaryXAxis.Title = "x-axis"


        'Set the axis labels to bold
        _barChart.PrimaryYAxis.Font = New Font("Arial", 8, FontStyle.Bold)
        _barChart.PrimaryXAxis.Font = New Font("Arial", 8, FontStyle.Bold)


        Dim i As Integer = 1
        For Each dataCollection In _dataCollection.DataCollection.Values

            'Create new serie
            series = _barChart.Model.NewSeries("series" & i.ToString, ChartSeriesType.Column)
            series.Text = series.Name

            'Lines between each series
            series.DrawColumnSeparatingLines = True

            'The pointIndex is neccesarry in order to keep track of how many points that have been 
            ' added to each serie. The Series.Style property (used to set column labels and tooltip)
            ' uses this pointIndex variable.
            pointIndex = 0
            For Each dataItem In dataCollection

                columnIndex = dataItem.Index

                'Add the point with the response to the correct column index
                series.Points.Add(columnIndex, dataItem.Value)

                displayText = dataItem.GroupDisplayText

                ' --------------------------------------------------------------------------------------------
                'Get max and min value of the columns in order to resize the chart correct in the x-direction
                ' --------------------------------------------------------------------------------------------
                'The first time in the loop we need a startvalue
                If firstTime Then
                    maxXValue = columnIndex
                    minXValue = columnIndex

                    maxYValue = dataItem.Value
                    minYValue = 0

                    firstTime = False
                End If

                'Get new max and min value for the x-axis
                If columnIndex < minXValue Then
                    minXValue = columnIndex
                End If
                If columnIndex > maxXValue Then
                    maxXValue = columnIndex
                End If

                'Get new max and min response
                If dataItem.Value < minYValue Then
                    minYValue = dataItem.Value
                End If
                If dataItem.Value > maxYValue Then
                    maxYValue = dataItem.Value
                End If

                'Since the labels are written everytime ach column is added to a serie we need to
                ' have a flag to only add the label once (the label text gets bold if written several times)
                If Not labelsHasBeenShown Then
                    _barChart.PrimaryXAxis.Labels.Add(New ChartAxisLabel(displayText, Color.Black, New Font("Arial", 8, FontStyle.Regular), columnIndex, "", ChartValueType.Custom))
                End If

                If _longLabelText Then
                    series.Styles(pointIndex).Text = "Column Label: " & dataItem.ColumnLabel
                Else
                    series.Styles(pointIndex).Text = dataItem.ColumnLabel
                End If


                'Tooltip text
                series.Styles(pointIndex).ToolTip = dataItem.ToolTip

                pointIndex += 1
            Next

            'Event that sets the color of each column
            AddHandler series.PrepareStyle, AddressOf SetColumnColor


            'View label text of all columns (if selected)
            series.Style.DisplayText = True
            series.Style.TextOrientation = ChartTextOrientation.Up
            series.Style.Font.Orientation = 270


            'Add serie to the chart
            _barChart.Series.Add(series)

            labelsHasBeenShown = True
            i += 1
        Next

        'Set correct tooltip format for all series
        SetToolTipFormat()

        _barChart.EndUpdate()

        'Resize the x-axis
        _barChart.PrimaryXAxis.Range.Max = maxXValue + 0.5
        _barChart.PrimaryXAxis.Range.Min = minXValue - 0.5

    End Sub

    Private Sub SetToolTipFormat()
        Dim s As ChartSeries

        For Each s In Me._barChart.Series
            Dim len As Integer = s.Points.Count
            Dim i As Integer = 0

            Do While i < len

                s.PointsToolTipFormat = "{2}"
                '
                'ToolTips Format Style
                '
                '					"{0}" - series Name
                '
                '					"{1}" - SeriesStyle.ToolTip;
                '
                '					"{2}" - SeriesStyle[i].ToolTip; - this is what we want.
                '
                '					"{3}" - series.Points[ index ].X;
                '
                '					"{4+i}" - series.Points[ index ].YValues[i];

                i += 1
            Loop
        Next s

    End Sub


    ''' <summary>
    ''' Sets a specific color of a defined bar in the chart
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="args"></param>
    ''' <remarks></remarks>
    Private Sub SetColumnColor(ByVal sender As Object, ByVal args As ChartPrepareStyleInfoEventArgs)
        Dim exseries As ChartSeries
        Dim tmpList As List(Of DataItem)
        Dim selectedcolor As Color

        exseries = CType(sender, ChartSeries)

        If Not exseries Is Nothing Then

            tmpList = _dataCollection.Item(exseries.Name)

            If Not tmpList Is Nothing Then
                Dim Xindex As Integer

                Xindex = CInt(exseries.Points(args.Index).X)

                selectedcolor = GetColorFromIndex(tmpList, Xindex)

                'Set color of column
                args.Style.Interior = New BrushInfo(GradientStyle.None, Color.Black, selectedcolor)

                args.Handled = True
            End If

        End If

    End Sub

    ''' <summary>
    ''' Gets a color in the color array from a index
    ''' </summary>
    ''' <param name="columnList"></param>
    ''' <param name="pointIndex"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function GetColorFromIndex(ByVal columnList As List(Of DataItem), ByVal pointIndex As Integer) As Color
        Dim selectedColor As Color = Aqua 'Set a default color
        Dim columnItem As DataItem

        For Each columnItem In columnList
            If columnItem.Index = pointIndex Then
                selectedColor = columnItem.Color
            End If
        Next

        Return selectedColor

    End Function

    Private Sub _updateButton_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles _updateButton.Click

        System.Windows.Forms.Cursor.Current = Cursors.WaitCursor

        _longLabelText = True
        RefreshChart()

        _dataCollection.GenerateData()
        UpdateChart()

        System.Windows.Forms.Cursor.Current = Cursors.Default


    End Sub

    Private Sub _dataButton_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles _dataButton.Click

        System.Windows.Forms.Cursor.Current = Cursors.WaitCursor

        _updateButton.Enabled = False

        _longLabelText = False
        RefreshChart()

        _dataCollection.GenerateData()
        UpdateChart()

        System.Windows.Forms.Cursor.Current = Cursors.Default

        _updateButton.Enabled = True

    End Sub

    Private Sub RefreshChart()

        'Clear chart and labels
        _barChart.Series.Clear()
        _barChart.PrimaryYAxis.Labels.Clear()
        _barChart.PrimaryXAxis.Labels.Clear()

        _dataCollection.Clear()
    End Sub

End Class
